--[[
	Main window, DPS window, and ability breakdown window
]]--
function CLog.MakeMainWindow()
	CLog.currentTab = "You"
	CLog.StartTime = 0
	CLog.Offset = 0
	CLog.PlayerBuffs = {}
	CLog.Check = false
	CLog.TotalDamage = 0
	CLog.TotalHealing = 0
	CLog.ScrollPosition = 0
	CLog.Alpha = 0
	CLog.Font = "EsoUI/Common/Fonts/univers57.otf"
	CLog.FontStyle = "soft-shadow-thin"
	CLog.XPGain = 0
	CLog.APGain = 0
	CLog.FadeTime = GetGameTimeMilliseconds()
	CLog.MainHidden = false
	CLog.TextHidden = false

	-- Make the main window
	local CL = CLog.Chain(WINDOW_MANAGER:CreateTopLevelWindow("CombatLog"))
		:SetDimensions(tonumber(CLog.settings.Width),tonumber(CLog.settings.Height))
		:SetAnchor(TOPLEFT,GuiRoot,TOPLEFT,tonumber(CLog.settings.SumX),tonumber(CLog.settings.SumY))
		:SetMovable(not CLog.settings.LockM)
		:SetMouseEnabled(true)
		:SetDrawLevel(DL_CONTROLS)
		:SetResizeHandleSize(MOUSE_CURSOR_RESIZE_NS)
		:SetHidden(CLog.settings.HiddenM)
		:SetClampedToScreen(true)
		:SetHandler( "OnMouseUp" , function(self) CLog.MouseUp(self) end)
		:SetHandler("OnMouseWheel",function(self,delta)
			if CLog.settings.Direction == "DOWN" then
				delta = delta * -1
			end
			CLog.ScrollPosition = CLog.ScrollPosition + delta
			if CLog.currentTab == "You" then
				CLog.ScrollWheel(CLog.settings.You)
			elseif CLog.currentTab == "Dam" then
				CLog.ScrollWheel(CLog.settings.Dam)
			elseif CLog.currentTab == "LT" then
				-- do nothing
			else
				CLog.ButtonToggle()
			end
		end)
	.__END

	CLog.MainWindow = CL

	-- give it a backdrop
	CL.bd = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_BD" , CL , CT_BACKDROP))
		:SetCenterColor(CLog.settings.BGColor[1],CLog.settings.BGColor[2],CLog.settings.BGColor[3],CLog.settings.BGColor[4])
		:SetEdgeColor(0,0,0,0)
		:SetAnchorFill(CL)
	.__END

	-- the textbox holder is a separate thing so it doesn't fade when the window does
	-- note that it doesn't have any dimensions; that's taken care of by the scrollchild
	local CT = CLog.Chain(WINDOW_MANAGER:CreateTopLevelWindow("CombatText"))
		:SetHidden(CLog.settings.HiddenM)
		:SetDrawLevel(DL_TEXT)
	.__END
	
	CLog.MainText = CT

	-- title bar
	CL.title = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Title", CL , CT_LABEL ))
		:SetFont("ZoFontGameOutline")
		:SetDimensions(CLog.settings.Width,40)
		:SetColor(255,255,255,1)
		:SetText("Combat Log Statistics")
		:SetAnchor(BOTTOM, CL ,TOP,0,-2)
	.__END

	CL.title.bd = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Title_BD", CL.title ,CT_BACKDROP))
		:SetCenterColor(0,0,0,1)
		:SetEdgeColor(0,0,0,0)
		:SetAnchorFill(CL.title)
	.__END

	----- CLOSE BUTTON on the title bar -----
	CL.title.close = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Close", CL.title , CT_BUTTON))
		:SetDimensions(20,20)
		:SetAnchor(TOPRIGHT, CL.title, TOPRIGHT,0,0)
		:SetFont("ZoFontGameOutline")
		:SetText("X")
		:SetHandler("OnClicked",function(self,but)
			CLog.settings.HiddenM = true
			CL:SetHidden(true)
			CT:SetHidden(true)
		end)
	.__END

	--"/esoui/art/buttons/swatchframe_down.dds"
	CL.title.close.bd = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Close_BD", CL.title.close ,CT_TEXTURE))
		:SetTexture("/esoui/art/buttons/swatchframe_down.dds")
		:SetColor(1,1,1,1)
		:SetAnchorFill(CL.title.close)
	.__END

	----- Back to TOP BUTTON -----
	CL.title.Top = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Top", CL.title , CT_BUTTON))
		:SetDimensions(20,20)
		:SetAnchor(RIGHT , CL.title , RIGHT,0,10)
		:SetHandler("OnClicked",function(self,but)
			CLog.ScrollPosition = 0
			if CLog.currentTab == "You" then
				CLog.ScrollWheel(CLog.settings.You)
			elseif CLog.currentTab == "Dam" then
				CLog.ScrollWheel(CLog.settings.Dam)
			else
				CLog.ButtonToggle()
			end
		end)
	.__END

	CL.title.Top.bd = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Top_BD" , CL.title.Top , CT_TEXTURE))
		:SetTexture("/esoui/art/buttons/dropbox_arrow_disabled.dds")
		:SetColor(1,1,1,1)
		:SetDimensions(20,20)
		:SetAnchor(TOPLEFT,CL.title.Top.bd,TOPLEFT,0,0)
	.__END

	----- ScrollChild: -----
	CL.scrollchild = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Scroll", CT, CT_SCROLL))
		:SetAnchorFill(CL)
		:SetHorizontalScroll(0)
	.__END

	----- TEXT BOX Lines -----
	local function createTextbox(num)
		local textbox= CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Textbox" .. num, CL.scrollchild, CT_LABEL))
			:SetFont(CLog.settings.Font .. "|" .. CLog.settings.FontSize .. "|" .. CLog.FontStyle)
			:SetText(CLog.settings.You[num])
			:SetDimensions(1000,CLog.settings.FontSize+1)
		.__END
		local Textbutton = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Textbutton" .. num, CL.scrollchild , CT_BUTTON))
			:SetDimensions(20,CLog.settings.FontSize+1)
			:SetFont(CLog.settings.Font .. "|" .. CLog.settings.FontSize .. "|" .. CLog.FontStyle)
			:SetAnchor(TOPLEFT, textbox, TOPLEFT, -20,0)
			:SetHidden(true)
		.__END
		local Textbuttonbd = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_TextbuttonBD" .. num, CL.scrollchild , CT_TEXTURE))
			:SetAnchorFill(Textbutton)
			:SetTexture("/esoui/art/buttons/swatchframe_down.dds")
			:SetHidden(true)
			:SetDrawLayer(11)
		.__END
		return textbox, Textbutton, Textbuttonbd
	end
	CLog.textbox = {}
	CLog.textbutton = {}
	CLog.textbuttonbd = {}
	for i = 1, CLog.settings.MaxLines , 1 do
		CLog.textbox[i], CLog.textbutton[i], CLog.textbuttonbd[i] = createTextbox(i)
	end	
	CLog.DirectionChange()

	----- TEXT BOX BUTTON Switchers -----
	CL.title.ButtonYou = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_You", CL.title , CT_BUTTON))
		:SetDimensions(40,20)
		:SetAnchor(LEFT, CL.title , LEFT ,0,10)
		:SetFont("ZoFontGameOutline")
		:SetText("All")
		:SetHandler("OnClicked",function(self,but)
			CLog.currentTab = "You"
			CLog.ScrollPosition = 0
			CLog.ScrollWheel(CLog.settings.You)
			CLog.DirectionChange()
			CLog.ButtonToggle()
		end)
	.__END

	CL.title.ButtonYou.bd  = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_You_BD", CL.title.ButtonYou ,CT_TEXTURE))
		:SetTexture("/esoui/art/buttons/swatchframe_down.dds")
		:SetColor(1,1,1,1)
		:SetAnchorFill(CL.title.ButtonYou)
	.__END

	CL.title.ButtonDam = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Dam", CL.title , CT_BUTTON))
		:SetDimensions(55,20)
		:SetAnchor(LEFT, CL.title.ButtonYou , RIGHT ,0,0)
		:SetFont("ZoFontGameOutline")
		:SetText("Health")
		:SetHandler("OnClicked",function(self,but)
			CLog.currentTab = "Dam"
			CLog.ScrollPosition = 0
			CLog.ScrollWheel(CLog.settings.Dam)
			CLog.DirectionChange()
			CLog.ButtonToggle()
		end)
	.__END

	CL.title.ButtonDam.bd  = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Dam_BD", CL.title.ButtonDam ,CT_TEXTURE))
		:SetTexture("/esoui/art/buttons/swatchframe_down.dds")
		:SetColor(1,1,1,1)
		:SetAnchorFill(CL.title.ButtonDam)
	.__END

	CL.title.ButtonAttack = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Attack", CL.title , CT_BUTTON))
		:SetDimensions(70,20)
		:SetAnchor(LEFT, CL.title.ButtonDam , RIGHT ,0,0)
		:SetFont("ZoFontGameOutline")
		:SetText("Attacks")
		:SetHandler("OnClicked",function(self,but)
			CLog.currentTab = "Attack"
			CLog.ScrollPosition = 0
			CLog.DirectionChange()
			CLog.ButtonToggle()
		end)
	.__END

	CL.title.ButtonAttack.bd  = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Attack_BD", CL.title.ButtonAttack ,CT_TEXTURE))
		:SetTexture("/esoui/art/buttons/swatchframe_down.dds")
		:SetColor(1,1,1,1)
		:SetAnchorFill(CL.title.ButtonAttack)
	.__END

	CL.title.ButtonHeal = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Heal", CL.title , CT_BUTTON))
		:SetDimensions(50,20)
		:SetAnchor(LEFT, CL.title.ButtonAttack , RIGHT ,0,0)
		:SetFont("ZoFontGameOutline")
		:SetText("Heals")
		:SetHandler("OnClicked",function(self,but)
			CLog.currentTab = "Heal"
			CLog.ScrollPosition = 0
			CLog.DirectionChange()
			CLog.ButtonToggle()
		end)
	.__END

	CL.title.ButtonHeal.bd  = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Heal_BD", CL.title.ButtonHeal ,CT_TEXTURE))
		:SetTexture("/esoui/art/buttons/swatchframe_down.dds")
		:SetColor(1,1,1,1)
		:SetAnchorFill(CL.title.ButtonHeal)
	.__END

	CL.title.ButtonLT = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_LT", CL.title , CT_BUTTON))
		:SetDimensions(40,20)
		:SetAnchor(LEFT, CL.title.ButtonHeal , RIGHT ,0,0)
		:SetFont("ZoFontGameOutline")
		:SetText("Stats")
		:SetHandler("OnClicked",function(self,but)
			CLog.currentTab = "LT"
			CLog.ScrollPosition = 0
			CLog.DirectionChange()
			CLog.ButtonToggle()
		end)
	.__END

	CL.title.ButtonLT.bd  = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_LT_BD", CL.title.ButtonLT ,CT_TEXTURE))
		:SetTexture("/esoui/art/buttons/swatchframe_down.dds")
		:SetColor(1,1,1,1)
		:SetAnchorFill(CL.title.ButtonLT)
	.__END

	----- TEXT BOX BUTTON Clear -----
	CL.title.labelClear = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Clear", CL.title, CT_BUTTON))
		:SetDimensions(50,20)
		:SetAnchor(RIGHT, CL.title.close , LEFT , -10,0)
		:SetFont("ZoFontCreditsText")
		:SetText("|c000000" .. "Clear" .. "|r")
		:SetHandler("OnClicked",function(self,but)
			CLog.settings.You = {"Cleared"}
			CLog.settings.Dam = {"Cleared"}
			CLog.settings.Attack = {}
			CLog.settings.Heal = {}
			CLog.ScrollPosition = 0
			if CLog.currentTab == "You" then
				CLog.ScrollWheel(CLog.settings.You)
			elseif CLog.currentTab == "Dam" then
				CLog.ScrollWheel(CLog.settings.Dam)
			elseif CLog.currentTab == "Attack" then
				CLog.ButtonToggle()
				CLog.textbox[1]:SetText("Cleared")
			elseif CLog.currentTab == "Heal" then
				CLog.ButtonToggle()
				CLog.textbox[1]:SetText("Cleared")
			end
			collectgarbage()
		end)
	.__END

	CL.title.labelClear.bd = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_Clear_BD", CL.title.labelClear ,CT_BACKDROP))
		:SetCenterColor(255,230,128,1)
		:SetEdgeColor(0,0,0,0)
		:SetAnchorFill(CL.title.labelClear)
	.__END

	----- Vertical Slider -----
	local tex = "/esoui/art/miscellaneous/scrollbox_elevator.dds"
	--CL.sliderV = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_SliderV" , CL , CT_SLIDER))
		--:SetDimensions(30,CL:GetHeight())
		--:SetMouseEnabled(true)
		--:SetThumbTexture(tex,tex,tex,30,10,1,0,1,0)
		--:SetValue(0)
		--:SetValueStep(2)
		--:SetMinMax(0,CL:GetHeight())
		--:SetHandler("OnValueChanged",function(self,value,eventReason)
			--CL.scrollchild:SetVerticalScroll(value*10)
			--CLog.Offset = value			
		--end)
		--:SetAnchor(RIGHT,CL,RIGHT,5,0)
	--.__END

	----- Horizontal Slider -----
	-- CL.sliderH = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_SliderH" , CL , CT_SLIDER))
		-- :SetDimensions(CLog.settings.Width,20)
		-- :SetMouseEnabled(true)
		-- :SetThumbTexture(tex,tex,tex,10,20,1,1,0,0)
		-- :SetValue(0)
		-- :SetMinMax(0,1000)
		-- :SetValueStep(1)
		-- :SetOrientation(1)
		-- :SetHandler("OnValueChanged",function(self,value,eventReason)
			-- d(value)
			-- _G["CombatLog_Scroll"]:SetHorizontalScroll(value)
		-- end)
		-- :SetAnchor(BOTTOMLEFT,CL,BOTTOMLEFT,0,22)
		-- :SetDrawLevel(2)
	-- .__END

	-- CL.sliderH.bd = CLog.Chain(WINDOW_MANAGER:CreateControl("CombatLog_SliderHBD" , CL.sliderH , CT_BACKDROP))
		-- :SetCenterColor(0,0,0,1)
		-- :SetEdgeColor(0,0,0,0)
		-- :SetAnchorFill(CL.sliderH)
		-- :SetDrawLevel(1)
	-- .__END

	-- Hide windows when compass frame hides
	if ZO_CompassFrame:IsHandlerSet("OnShow") then
		local oldHandler = ZO_CompassFrame:GetHandler("OnShow")
		ZO_CompassFrame:SetHandler("OnShow", function(...)
			oldHandler(...)
			CL:SetHidden(CLog.MainHidden)
			CT:SetHidden(CLog.TextHidden)
			if CLog.DPSWindow then CLog.DPSWindow:SetHidden(CLog.DPSHidden) end
		end)
	else
		ZO_CompassFrame:SetHandler("OnShow", function(...)
			CL:SetHidden(CLog.MainHidden)
			CT:SetHidden(CLog.TextHidden)
			if CLog.DPSWindow then CLog.DPSWindow:SetHidden(CLog.DPSHidden) end
		end)
	end
	if ZO_CompassFrame:IsHandlerSet("OnHide") then
		oldHandler = ZO_CompassFrame:GetHandler("OnHide")
		ZO_CompassFrame:SetHandler("OnHide", function(...)
			oldHandler(...)
			CLog.MainHidden = CL:IsHidden()
			CL:SetHidden(true)
			CLog.TextHidden = CT:IsHidden()
			CT:SetHidden(true)
			if CLog.DPSWindow then
				CLog.DPSHidden = CLog.DPSWindow:IsHidden()
				CLog.DPSWindow:SetHidden(true)
			end
		end)
	else
		ZO_CompassFrame:SetHandler("OnHide", function(...)
			CLog.MainHidden = CL:IsHidden()
			CL:SetHidden(true)
			CLog.TextHidden = CT:IsHidden()
			CT:SetHidden(true)
			if CLog.DPSWindow then
				CLog.DPSHidden = CLog.DPSWindow:IsHidden()
				CLog.DPSWindow:SetHidden(true)
			end
		end)
	end
end

--[[--
 * Function to record frame positions in saved variables
 ]]--
function CLog.MouseUp(self)
	-- assign the LEFT point of the window
	local left = self:GetLeft()

	-- assign the TOP point of the window
	local top = self:GetTop()

	-- assign Height and Width of window
	local Height = self:GetHeight()
	local Width = self:GetWidth()

	-- Save the position information for LEFT and TOP to the variables file
	CLog.settings.Height = Height
	CLog.settings.Width = Width
	CLog.settings.SumX = left
	CLog.settings.SumY = top

	-- Resize Title
	local title = self.title
--	local slider = _G["CombatLog_SliderH"]
	title:SetDimensions(Width,40)
--	slider:SetDimensions(Width,20)
end

--[[--
 * Function scroll labels
 ]]--
function CLog.ScrollWheel(Dtable)
	if CLog.ScrollPosition < 0 then
		CLog.ScrollPosition = 0
	end
	--if CLog.settings.MaxLines > #Dtable then
		--CLog.ScrollPosition = 0
	if CLog.ScrollPosition > #Dtable - 1 then
		CLog.ScrollPosition = #Dtable - 1
	end
	for i = 1, CLog.settings.MaxLines do
		if Dtable[CLog.ScrollPosition + i] ~= nil then
			CLog.textbox[i]:SetText(Dtable[CLog.ScrollPosition + i])
		else
			CLog.textbox[i]:SetText("")
		end
	end
end

--Show or Hides Buttons within the scroll, depending on which display is selected
function CLog.ButtonToggle()
	local dtable = {}
	-- start by hiding all the buttons
	for i = 1, CLog.settings.MaxLines do
		CLog.textbutton[i]:SetHidden(true)
		CLog.textbuttonbd[i]:SetHidden(true)
	end
	-- the buttons should only show for the Attack and Heal displays
	if CLog.currentTab == "Attack" then
		dtable = CLog.settings.Attack
	elseif CLog.currentTab == "Heal" then
		dtable = CLog.settings.Heal
	else
		--- ...so if we're not in one of those, go away
		return
	end
	-- sort the table of abilities by total damage/healing done
	local Display, Match = CLog.SortTable(dtable)
	-- sanity check the scroll position
	if CLog.ScrollPosition < 0 then
		CLog.ScrollPosition = 0
	end
	if CLog.ScrollPosition > #Display - 1 then
		CLog.ScrollPosition = #Display - 1
	end
	-- if we have more abilities to display than we have lines to display them in, truncate the table to only show the top abilities
	if (#Display > CLog.settings.MaxLines) then
		for i = CLog.settings.MaxLines + 1, #Display do
			Display[i] = nil
			Match[i] = nil
		end
	end

	-- for every line...
	for i = 1, CLog.settings.MaxLines do
		-- set the text of the line
		CLog.textbox[i]:SetText("")
		CLog.textbox[i]:SetText(Display[i+CLog.ScrollPosition])
		-- if the line is not blank,...
		if (Display[i+CLog.ScrollPosition] ~= nil) and not (i+CLog.ScrollPosition > #Display) and not (i+CLog.ScrollPosition == #Display) then
			-- show the button, put a number on it, and give the button something to do
			if (CLog.settings.Direction == "DOWN") then
				CLog.textbutton[i]:SetHidden(false)
				CLog.textbuttonbd[i]:SetHidden(false)
				CLog.textbutton[i]:SetText(i+CLog.ScrollPosition .. ".")
				CLog.textbutton[i]:SetHandler("OnClicked",function(self,but)
					CLog.AbilityWindow.label:SetText("")
					CLog.AbilityWindow.label:SetText(Match[i+CLog.ScrollPosition])
					if CLog.AbilityWindow.label:GetWidth() == 0 then
						CLog.AbilityWindow:SetDimensions(300,300)
					else
						CLog.AbilityWindow:SetDimensions(CLog.AbilityWindow.label:GetWidth()+20, CLog.AbilityWindow.label:GetHeight() + 5)
					end
					CLog.AbilityWindow.bd:SetAnchorFill(CLog.AbilityWindow)
					CLog.AbilityWindow:SetHidden(false)
				end)
			else
				i = i + CLog.ScrollPosition
				CLog.textbutton[#Display+1-i]:SetHidden(false)
				CLog.textbuttonbd[#Display+1-i]:SetHidden(false)
				CLog.textbutton[#Display+1-i]:SetText(i-CLog.ScrollPosition.. ".")
				CLog.textbutton[#Display+1-i]:SetHandler("OnClicked",function(self,but)
					CLog.AbilityWindow.label:SetText("")
					CLog.AbilityWindow.label:SetText(Match[#Display-i+CLog.ScrollPosition])
					if CLog.AbilityWindow.label:GetWidth() == 0 then
						CLog.AbilityWindow:SetDimensions(300,300)
					else
						CLog.AbilityWindow:SetDimensions(CLog.AbilityWindow.label:GetWidth()+20, CLog.AbilityWindow.label:GetHeight() + 5)
					end
					CLog.AbilityWindow.bd:SetAnchorFill(CLog.AbilityWindow)
					CLog.AbilityWindow:SetHidden(false)
				end)
			end
		end
	end
end

--[[-----------------------------------------------
	Changes Direction of You and Dam windows
 --]]-----------------------------------------------
function CLog.DirectionChange()
	if (CLog.currentTab == "You") or (CLog.currentTab == "Dam") or (CLog.currentTab == "LT") then
		if (CLog.settings.Direction == "UP") then
			for i = 1, CLog.settings.MaxLines , 1 do
				if i == 1 then
					CLog.textbox[i]:SetAnchor(BOTTOMLEFT, CLog.MainWindow.scrollchild, BOTTOMLEFT, 0,0)
				else
					CLog.textbox[i]:SetAnchor(BOTTOMLEFT, CLog.textbox[i-1], TOPLEFT, 0,0)
				end
			end	
		elseif (CLog.settings.Direction == "DOWN") then
			for i = 1, CLog.settings.MaxLines , 1 do
				if i == 1 then
					CLog.textbox[i]:SetAnchor(TOPLEFT, CLog.MainWindow.scrollchild, TOPLEFT, 0,0)
				else
					CLog.textbox[i]:SetAnchor(TOPLEFT, CLog.textbox[i-1], BOTTOMLEFT, 0,0)
				end
			end
		end
	elseif (CLog.currentTab == "Attack") or (CLog.currentTab == "Heal") then
		if (CLog.settings.Direction == "UP") then
			for i = 1, CLog.settings.MaxLines , 1 do
				if i == 1 then
					CLog.textbox[i]:SetAnchor(BOTTOMLEFT, CLog.MainWindow.scrollchild, BOTTOMLEFT, 20,0)
				else
					CLog.textbox[i]:SetAnchor(BOTTOMLEFT, CLog.textbox[i-1], TOPLEFT, 0,0)
				end
			end	
		elseif (CLog.settings.Direction == "DOWN") then
			for i = 1, CLog.settings.MaxLines , 1 do
				if i == 1 then
					CLog.textbox[i]:SetAnchor(TOPLEFT, CLog.MainWindow.scrollchild, TOPLEFT, 20,0)
				else
					CLog.textbox[i]:SetAnchor(TOPLEFT, CLog.textbox[i-1], BOTTOMLEFT, 0,0)
				end
			end
		end
	end
end

--[[----------------------------------------------------------
	Keybind Function
 ]]-----------------------------------------------------------
function CLog.ToggleMain()
	local main = CLog.MainWindow
	local text = CLog.MainText

	if main and text and (main:IsHidden() == true) then
		CLog.settings.HiddenM = false
		main:SetHidden(false)
		text:SetHidden(false)
		CLog.FadeTime = GetGameTimeMilliseconds()
	elseif main and text and (main:IsHidden() == false) then
		CLog.settings.HiddenM = true
		main:SetHidden(true)
		text:SetHidden(true)
	end
end

-- takes a new line of text and inserts it into the log, then updates the window 
-- to display it if needed
function CLog.AddTextToLog(whichLog, newLine)
	if whichLog ~= "You" and whichLog ~= "Dam" then return end

	-- add timestamp if needed
	if CLog.settings.Timestamp == true then
		newLine = "[" .. GetTimeString() .. "]" .. newLine
	end

	-- log the new line
	if whichLog == "You" then
		table.insert(CLog.settings.You, 1, newLine)
		CLog.WriteToChat(newLine)
	elseif whichLog == "Dam" then
		table.insert(CLog.settings.Dam, 1, newLine)
	end

	-- trim old data from the log if needed
	if CLog.settings.MaxLinesSaved ~= 0 or nil then
		if #CLog.settings.You > CLog.settings.MaxLinesSaved then
			for i = CLog.settings.MaxLinesSaved + 1, #CLog.settings.You do
				CLog.settings.You[i] = nil
			end
		end
		if #CLog.settings.Dam > CLog.settings.MaxLinesSaved then
			for i = CLog.settings.MaxLinesSaved + 1, #CLog.settings.Dam do
				CLog.settings.Dam[i] = nil
			end
		end
	end

	-- refresh the window if needed
	if CLog.currentTab == whichLog then
		-- restart the fade timer
		CLog.FadeTime = GetGameTimeMilliseconds()
		-- redisplay the data
		if CLog.currentTab == "You" then
			for i = 1, CLog.settings.MaxLines do
				if CLog.settings.You[CLog.ScrollPosition + i] ~= nil then
					CLog.textbox[i]:SetText(CLog.settings.You[CLog.ScrollPosition + i])
				end
			end
		elseif CLog.currentTab == "Dam" then
			for i = 1, CLog.settings.MaxLines do
				if CLog.settings.Dam[CLog.ScrollPosition + i] ~= nil then
					CLog.textbox[i]:SetText(CLog.settings.Dam[CLog.ScrollPosition + i])
				end
			end
		end
	end
end

function CLog.MakeDPSWindow()
	--Setup variables
	CLog.DPSMeter	= 0
	CLog.HPSMeter	= 0
	CLog.Target 		= {}
	CLog.Damage 		= {}
	CLog.Heal 		= {}
	CLog.State 		= false
	CLog.FTime 		= 0
	CLog.DPSHidden	= false

	-- STAND ALONE DPS METER
	local DPS = CLog.Chain(WINDOW_MANAGER:CreateTopLevelWindow("DPS"))
		--frame:SetDimensions(DPS_Label:GetWidth(),DPS_Label:GetHeight())
		:SetHidden(CLog.settings.HiddenD)
		:SetAnchor(TOPLEFT,GuiRoot,TOPLEFT,CLog.settings.SumXDPS,CLog.settings.SumYDPS)
		:SetMovable(not CLog.settings.LockD)
		:SetClampedToScreen(true)
		:SetMouseEnabled(true)
		:SetHandler("OnMouseUp", function(self) CLog.MouseUpDPS( self ) end)
		--:SetResizeToFitDescendents(true)
	.__END
	
	CLog.DPSWindow = DPS

	DPS.BG = WINDOW_MANAGER:CreateControlFromVirtual("DPS_BG", DPS, "ZO_ThinBackdrop")

	DPS.Close = CLog.Chain(WINDOW_MANAGER:CreateControl("DPS_Close" , DPS , CT_BUTTON))
		:SetDimensions(20,20)
		:SetAnchor(TOPRIGHT, DPS, TOPRIGHT, 0,0)
		:SetFont(CLog.Font .. "|" .. CLog.settings.FontSize .. "|" .. CLog.FontStyle)
		:SetText("X")
		:SetHandler("OnClicked", function(self) CLog.ToggleDPS() end)
	.__END

	DPS.LabelTex = CLog.Chain(WINDOW_MANAGER:CreateControl("DPS_LabelTex" , DPS , CT_LABEL))
		:SetAnchor(TOPLEFT, DPS, TOPLEFT, 5, 20)
		:SetFont(CLog.Font .. "|" .. CLog.settings.FontSize .. "|" .. CLog.FontStyle)
		:SetText("DPS:" .. "\n" .. "Total Damage:" .. "\n" .. "HPS:" .. "\n" .. "Total Heal:" .. "\n" .. "Total Time:")
	.__END

	DPS.LabelVal = CLog.Chain(WINDOW_MANAGER:CreateControl("DPS_LabelVal" , DPS , CT_LABEL))
		:SetAnchor(LEFT, DPS.LabelTex, RIGHT, 5, 0)
		:SetFont(CLog.Font .. "|" .. CLog.settings.FontSize .. "|" .. CLog.FontStyle)
		:SetText("0\n0\n0\n0\n0")
	.__END
	
	DPS.SendToChat = CLog.Chain(WINDOW_MANAGER:CreateControl("DPS_SendToChat", DPS, CT_BUTTON))
		:SetDimensions(100,30)
		:SetAnchor(TOPLEFT, DPS.LabelTex, BOTTOMLEFT, 0, 5)
		:SetFont(CLog.Font .. "|" .. CLog.settings.FontSize .. "|" .. CLog.FontStyle)
		:SetText("Send to Group Chat")
		:SetHandler("OnClicked", function(self) CLog.SendDPSToChat() end)
	.__END

	DPS:SetDimensions(DPS.LabelTex:GetWidth() + DPS.LabelVal:GetWidth() + 25, DPS.LabelTex:GetHeight() + DPS.Close:GetHeight() + DPS.SendToChat:GetHeight() + 5)
	
	DPS.StartTime = 0
end

function CLog.ToggleDPS()
	local dps = CLog.DPSWindow

	if dps:IsHidden() == true then
		CLog.settings.HiddenD = false
		CLog.DPSWindow:SetHidden(false)
	else
		CLog.settings.HiddenD = true
		CLog.DPSWindow:SetHidden(true)
	end
end

function CLog.SendDPSToChat()
	local stats = CLog.DPSWindow.LabelVal:GetText()
	local getStat = string.gmatch(stats, "[%.%w]+")
	local dps = getStat()
	local damage = getStat()
	local hps = getStat()
	local healing = getStat()
	local fighttime = getStat()
	local charname = GetUnitName("player")
	local infostring = "Combat Log Statistics for " .. charname .. ":  "
	infostring = infostring .. "DPS: " .. dps .. "  HPS: " .. hps .. "  "
	infostring = infostring .. "Damage: " .. damage .. "  Heals: " .. healing .. "  "
	infostring = infostring .. "Fight time: " .. fighttime .. " sec."
	StartChatInput(infostring, CHAT_CHANNEL_PARTY)
end

--[[--
 * Function to record frame positions in saved variables
 ]]--
function CLog.MouseUpDPS(self)
	-- assign the LEFT point of the window
	local left = self:GetLeft()

	-- assign the TOP point of the window
	local top = self:GetTop()

	-- Save the position information for LEFT and TOP to the variables file
	CLog.settings.SumXDPS = left
	CLog.settings.SumYDPS = top
end

function CLog.MakeAbilityWindow()
	-- ABILITY BREAKDOWN WINDOW
	local AF = CLog.Chain(WINDOW_MANAGER:CreateTopLevelWindow("AbilityFrame"))
		:SetDimensions(200,200)
		:SetHidden(true)
		:SetAnchor(CENTER,GuiRoot,CENTER,0,0)
		:SetMovable(true)
		:SetMouseEnabled(true)
		:SetResizeHandleSize(MOUSE_CURSOR_RESIZE_NS)
	.__END

	AF.bd = CLog.Chain(WINDOW_MANAGER:CreateControl("AbilityFrame_BD" , AF , CT_BACKDROP))
		:SetCenterColor(0,0,0,.5)
		:SetEdgeColor(0,0,0,0)
		:SetAnchorFill(AF)
	.__END

	--[[
	AF.title = CLog.Chain(WINDOW_MANAGER:CreateControl("AbilityFrame_Title", AF , CT_LABEL ))
		:SetFont("ZoFontGameOutline")
		:SetDimensions(CLog.settings.Width,20)
		:SetColor(255,255,255,1)
		:SetText("Combat Log Statistics")
		:SetAnchor(BOTTOM, AF ,TOP,0,0)
	.__END

	AF.title.bd = CLog.Chain(WINDOW_MANAGER:CreateControl("AbilityFrame_Title_BD", AF.title ,CT_BACKDROP))
		:SetCenterColor(0,0,0,1)
		:SetEdgeColor(0,0,0,0)
		:SetAnchorFill(AF.title)
	.__END	
	]]--

	AF.label = CLog.Chain(WINDOW_MANAGER:CreateControl("AbilityFrame_Label", AF , CT_LABEL ))
		-- :SetDimensions(CLog.settings.Width,CLog.settings.Height)
		:SetFont(CLog.Font .. "|" .. CLog.settings.FontSize .. "|" .. CLog.FontStyle)
		:SetColor(255,255,255,1)
		:SetAnchor(TOPLEFT, AF, TOPLEFT, 0,0)
	.__END

	----- CLOSE BUTTON -----
	AF.close = CLog.Chain(WINDOW_MANAGER:CreateControl("AbilityFrame_Close", AF, CT_BUTTON))
		:SetDimensions(20,20)
		:SetAnchor(TOPRIGHT, AF ,TOPRIGHT,0,0)
		:SetFont("ZoFontGameOutline")
		:SetText("X")
		:SetHandler("OnClicked",function(self,but) AF:SetHidden(true) end)
	.__END

	-- "/esoui/art/buttons/swatchframe_down.dds"
	AF.close.bd = CLog.Chain(WINDOW_MANAGER:CreateControl("AbilityFrame_Close_BD", AF.close ,CT_TEXTURE))
		:SetTexture("/esoui/art/buttons/swatchframe_down.dds")
		:SetColor(1,1,1,1)
		:SetAnchorFill(AF.close)
	.__END
	
	CLog.AbilityWindow = AF
end