--[[----------------------------------------------------------
	Combat Log Statistics
		AddonAuthor: Zeakfury	
	Thanks to Atropos (FTC) and Errc for much of the code here
	patched by katkat42 to work with the Vet Crypt of Hearts patch
	patched by katkat42 to work with the Update 3 (dyes and guilds) patch
  ]]----------------------------------------------------------

-- Create Global

CLog = {}
CLog.version = "3.0.5.1"
CLog.Starter = false
CLog.LAM = LibStub("LibAddonMenu-2.0")

--[[----------------------------------------------------------
	INITIALIZATION
  ]]----------------------------------------------------------

local function OnAddOnLoaded(eventCode, addOnName)
	if addOnName ~= "CombatLogStatistics" then return end
	EVENT_MANAGER:UnregisterForEvent("CombatLogStatistics", ON_ADD_ON_LOADED)
	CLog.Default	= {
		["You"] = {"Loaded"},
		["Dam"] = {"Loaded"},
		["Attack"] = {},
		["Heal"] = {},
		["SumX"] = 0,
		["SumY"] = 0,
		["Width"] = 400,
		["Height"] = 200,
		["SumXDPS"] = 0,
		["SumYDPS"] = 0,
		["MaxLines"] = 15,
		["MaxLinesSaved"] = 100,
		["YouColor"] = "|c00ff00",
		["PetColor"] = "|c98eab8",
		["GroupColor"] = "|c0000ff",
		["AllyColor"] = "|c80ffff",
		["NPCColor"] = "|cFFFF00",
		["EnemyColor"] = "|cff0000",
		["SpecialColor"] = "|cead585",
		["GoodBuffColor"] = "|cFF00FF",
		["BadBuffColor"] = "|c8F00FF",
		["FoodBuffColor"] = "|cFF7F00",
		["HealColor"] = "|c73bdff",
		["GenericColor"] = "|cafb9cc",
		["PhysicalColor"] = "|cffe29d",
		["FireColor"] = "|cff2e2b",
		["ShockColor"] = "|c78bfff",
		["OblivionColor"] = "|c000030",
		["ColdColor"] = "|c3b55ff",
		["EarthColor"] = "|cc66300",
		["MagicColor"] = "|cd37fff",
		["DrownColor"] = "|cff72bc",
		["DiseaseColor"] = "|c24c603",
		["PoisonColor"] = "|c76dd14",
		["Timestamp"] = true,
		["ShowHeals"] = true,
		["ShowDamage"] = true,
		["NameColor"] = true,
		["AbilityColor"] = true,
		["BuffColor"] = true,
		["ShowPower"] = true,
		["ShowBG"] = false,
		["FadesWithChat"] = true,
		["FadeTextToo"] = true,
		["FadeDPSWithChat"] = true,
		["FadeTimer"] = 10,
		["BGColor"] = {0,0,0,.5},
		["XP"] = true,
		["AP"] = true,
		["Buffs"] = true,
		["Chat"] = "None",
		["FontSize"] = 18,
		["Font"] = "EsoUI/Common/Fonts/univers57.otf",
		["HiddenM"] = false,
		["HiddenD"] = true,
		["LockM"] = false,
		["LockD"] = false,
		["Direction"] = "DOWN",
		["ToggleTracking"] = true,
		["Buffer"] = true,
		["BufferSpeed"] = 0.1,
		["apiVersion"] = GetAPIVersion(),
		["ShowSkills"] = {
			[8] = {
				[1] = true,	-- alchemy
				[2] = true,	-- blacksmithing
				[3] = true,	-- clothing
				[4] = true,	-- enchanting
				[5] = true,	-- provisioning
				[6] = true	-- woodworking
			},
			[7] = { [1] = true },	-- XP: is this still used?
			[6] = {
				[1] = true,	-- assault
				[2] = true,	-- support
				[3] = true	-- emperor
			},
			[5] = {
				[1] = true,	-- dark brotherhood
				[2] = true,	-- fighters guild
				[3] = true,	-- mages guild
				[4]	= true,	-- thieves guild
				[5]	= true	-- undaunted
			},
			[4] = {
				[1] = true,	-- Legerdemain (or soul trap)
				[2] = true,	-- soul trap
				[3] = true	-- vampire/werewolf
			},
			[3]	= {	
				[1] = true,	-- light armor
				[2] = true,	-- medium armor
				[3] = true	-- heavy armor
			},
			[2] = {	
				[1] = true,	-- Two-handed
				[2] = true,	-- One hand and shield
				[3] = true,	-- dual wield
				[4] = true,	-- bow
				[5] = true,	-- destruction staff
				[6] = true	-- restoration staff
			},
			[1]	= {	
				[1] = true,	-- Class skill line 1
				[2] = true,	-- Class skill line 2
				[3] = true	-- Class skill line 3
			}
		},
		["Stats"] = {
			["Death"] = 0,
			["THeal"] = 0,
			["TDam"] = 0,
			["IncDam"] = 0,
			["IncHeal"] = 0,
			["MaxDam"] = 0,
			["MaxHeal"] = 0,
			["MaxIncDam"] = 0,
			["MaxIncHeal"] = 0,
			["MaxCritDam"] = 0,
			["MaxCritHeal"] = 0,
			["FTime"] = 1,
			["Kills"] = 0,
		}
	}

	-- Load saved variables
	CLog.settings = ZO_SavedVars:New( "FCLText" , 3 , nil , CLog.Default , nil )
	if CLog.settings.Chat == true then CLog.settings.Chat = "First Tab" end
	if CLog.settings.Chat == false then CLog.settings.Chat = "None" end

	-- Get Character Info before we configure the menu
	CLog.CharacterStats()
	CLog.MakeMainWindow()
	CLog.MakeDPSWindow()
	CLog.MakeAbilityWindow()
	-- is this a long enough gap?
	CLog.InitMenu()
	-- can't touch the chat system until player activated
	EVENT_MANAGER:RegisterForEvent("CLSChat", EVENT_PLAYER_ACTIVATED, function(...) CLog.InitChatTab(...) end)

	-- Register event listeners
	EVENT_MANAGER:RegisterForEvent( "CLS" , EVENT_COMBAT_EVENT , CLog.NewCombatEvent )
	EVENT_MANAGER:RegisterForEvent( "CLS" , EVENT_PLAYER_COMBAT_STATE , CLog.InCombat)
	EVENT_MANAGER:RegisterForEvent( "CLS" , EVENT_EFFECT_CHANGED , CLog.NewBuff )
	-- EVENT_MANAGER:RegisterForEvent( "CLS" , EVENT_FINESSE_RANK_CHANGED , CLog.Finesse )
	EVENT_MANAGER:RegisterForEvent( "CLS" , EVENT_EXPERIENCE_UPDATE , CLog.XPUpdate )
	--EVENT_MANAGER:RegisterForEvent( "CLS" , EVENT_QUEST_COMPLETE_EXPERIENCE , CLog.QuestExp )
	EVENT_MANAGER:RegisterForEvent( "CLS", EVENT_SKILL_LINE_ADDED , CLog.CharacterStats )
	EVENT_MANAGER:RegisterForEvent( "CLS" , EVENT_VETERAN_POINTS_UPDATE , CLog.XPUpdate )
	EVENT_MANAGER:RegisterForEvent( "CLS" , EVENT_ALLIANCE_POINT_UPDATE  , CLog.NewAP )
	EVENT_MANAGER:RegisterForEvent( "CLS" , EVENT_SKILL_XP_UPDATE , CLog.SkillPointsUpdate)
	EVENT_MANAGER:RegisterForEvent( "CLS" , EVENT_PLAYER_DEAD , CLog.Dead )

	-- Register Slash Commands
	SLASH_COMMANDS["/cls"] = CLog.SlashCommands

	-- Populate buffs for the player
	CLog.GetBuffs( 'player' )

	-- don't start the OnUpdate routine running until we're done setting up
	CLog.Starter = true
end

EVENT_MANAGER:RegisterForEvent("CombatLogStatistics", EVENT_ADD_ON_LOADED, OnAddOnLoaded)

-- Register keybindings
ZO_CreateStringId("SI_BINDING_NAME_DISPLAY_COMBAT_LOG", "Display Combat Log")
ZO_CreateStringId("SI_BINDING_NAME_DISPLAY_DPS_LOG", "Display DPS Meter")


--[[---------------------------------------------------------------
 * A handy chaining function for quickly setting up UI elements
 ]]----------------------------------------------------------------
function CLog.Chain( object )
	-- Setup the metatable
	local T = {}
	setmetatable( T , { __index = function( self , func )
		-- Know when to stop chaining
		if func == "__END" then	return object end
		-- Otherwise, add the method to the parent object
		return function( self , ... )
			assert( object[func] , func .. " missing in object" )
			object[func]( object , ... )
			return self
		end
	end })

	-- Return the metatable
	return T
end

--[[
  * Adds global /CLS to show CombatLog
 ]]--
function CLog.SlashCommands(text)
	text = text:upper()
	if (text == "HELP") or (text == "") then
		d("|cff180CLSS Help Guide|r")
		d("|c00ffffIf you are wondering why you cannot see the Target that hit you or the Ability they hit you with and only the damage that was done to you, the reason is because ZOS decided to block these in the API. Please show support for getting these unblocked.|r")
		d("|cff180f/CLS Main|r to open up Main window")
		d("|cff180f/CLS DPS|r to open up DPS window")
		d("|cff180f/CLS Settings|r to open up Settings window")
	elseif text == "MAIN" then
		CLog.ToggleMain()
	elseif text == "DPS" then
		CLog.ToggleDPS()
	elseif text == "SETTINGS" then
		SLASH_COMMANDS["/clssettings"]()
	end	
end

--[[
  * Handles Skill XP events and adds them to the combat damage table.
  * Runs on the EVENT_SKILL_XP_UPDATE.
  altered by katkat42 to account for new EVENT_SKILL_XP_UPDATE return values, and to display correct skill titles
 ]]--
function CLog.SkillPointsUpdate(eventCode, skillCategory, skillType, reason, rank, previousXP, currentXP)
    local diff = 0
    local inLevel = 0
    local display = ""

	-- if this isn't really an update, go away
    if (CLog.SkillsXP[skillCategory][skillType][3] == nil) or (CLog.SkillsXP[skillCategory][skillType][3] >= currentXP) then return end
	-- if we've requested not to update this particular skill, go away
    if (CLog.settings.ShowSkills[skillCategory][skillType] == false) then return end

    diff = currentXP - previousXP
	local lastRankXP, nextRankXP = GetSkillLineXPInfo(skillCategory, skillType)
	local curLevelXP = nextRankXP - lastRankXP
	local levelProg = currentXP - lastRankXP
    local percent = string.format( "%.2f" , (levelProg/curLevelXP)*100)
	skillName = GetSkillLineInfo(skillCategory, skillType)
    display = "|cc8ff3a+" .. diff .. " " .. skillName ..  " (" .. percent .. "%)"

	CLog.SkillsXP[skillCategory][skillType] = {lastRankXP, nextRankXP, currentXP}

	CLog.AddTextToLog("You", display)
end

--[[
	OLD CODE
  * Handles experience events and adds them to the combat damage table.
  * Runs on the EVENT_EXPERIENCE_UPDATE and EVENT_VETERAN_POINTS_UPDATE listeners.
  altered by katkat42 to account for new event return values, and to display XP/VP properly
 ]]--
function CLog.XPUpdate( eventCode, unitTag, currentExp, maxExp, reason )
	--d("NewExp // " .. currentExp)
	local display = ""

	-- Bail if it's not earned by the player
	if ( unitTag ~= "player" ) then return end

	-- Don't display experience for level 50 characters
	--if ( eventCode == EVENT_EXPERIENCE_UPDATE and GetUnitLevel('player') == 50 ) then return end

	-- Check whether it's VP or EXP
	local isveteran = ( eventCode == EVENT_VETERAN_POINTS_UPDATE ) and true or false

	-- Ignore finesse bonuses, they will get rolled into the next reward
	if ( reason == PROGRESS_REASON_FINESSE ) then return end

	-- Get the old experience
	local base = isveteran and CLog.VetXP or CLog.XP

	-- Calculate the difference
	local diff 	= currentExp - base
	--d(currentExp.." - "..base.." = "..diff)

	-- Record Gains for current Game-play
	CLog.XPGain = CLog.XPGain + diff

	-- Ignore zero experience rewards
	if ( diff == 0 ) or ( diff < 0 ) then return end

	-- Only show if wanted
	if ( CLog.settings.XP == false ) then return end

	-- Update the recorded experience
	if ( isveteran ) then
		display = "|c75d3ff+" .. diff .. " Veteran Points"
		CLog.VetXP = currentExp
	else
		display = "|c75d3ff+" .. diff .. " Experience Points"
		CLog.XP = currentExp
	end

	CLog.AddTextToLog("You", display)
end

--[[
  * Handles alliance point gains.
  * Runs on the EVENT_ALLIANCE_POINT_UPDATE listener.
 ]]--
function CLog.NewAP( eventCode, alliancePoints, playSound, difference )
	-- Record Gains for current Game-play
	CLog.APGain = CLog.APGain + difference

	-- ONly if player wants
	if ( CLog.settings.AP == false ) then return end

	-- Ignore tiny AP rewards
	if ( difference  < 5 ) then return end

	display = "|cff8f41+" .. difference .. " Alliance Points"

	CLog.AddTextToLog("You", display)
end

--[[
	* Gets Current Stats for Character
--]]
function CLog.CharacterStats()
	CLog.CName = GetUnitName( 'player' )
	CLog.XP = GetUnitXP('player')
	CLog.VetXP = GetUnitVeteranPoints('player')
	-- GetSkillLineXPInfo() returns lastrank, nextrank, current
	CLog.SkillsXP = {
		[8] = {	
			[1] = {GetSkillLineXPInfo(8,1)}, 	-- Alchemy
			[2] = {GetSkillLineXPInfo(8,2)}, 	-- Blacksmithing
			[3] = {GetSkillLineXPInfo(8,3)},	-- Clothing
			[4] = {GetSkillLineXPInfo(8,4)},	-- Enchanting
			[5] = {GetSkillLineXPInfo(8,5)},	-- Provisioning
			[6] = {GetSkillLineXPInfo(8,6)}		-- Woodworking
		},
		[7] = { [1] = { GetSkillLineXPInfo(7,1) } },	-- PlayerXP
		[6] = {
			[1] = {GetSkillLineXPInfo(6,1)},	-- Assault
			[2] = {GetSkillLineXPInfo(6,2)},	-- Support
			[3] = {GetSkillLineXPInfo(6,3)}		-- Emperor
		},
		[5] = {	
			[1] = {GetSkillLineXPInfo(5,1)},	-- Dark Brotherhood
			[2] = {GetSkillLineXPInfo(5,2)},	-- Fighters Guild
			[3] = {GetSkillLineXPInfo(5,3)},	-- Mages Guild
			[4] = {GetSkillLineXPInfo(5,4)},	-- Thieves Guild
			[5]	= {GetSkillLineXPInfo(5,5)}		-- Undaunted
		},
		[4] = {
			[1] = {GetSkillLineXPInfo(4,1)},	-- Legerdemain
			[2] = {GetSkillLineXPInfo(4,2)},	-- Soul
			[3] = {GetSkillLineXPInfo(4,3)}		-- Vamp/WW
		},	
		[3]	= {	
			[1] = {GetSkillLineXPInfo(3,1)},	-- Light Armor
			[2] = {GetSkillLineXPInfo(3,2)},	-- Medium Armor
			[3] = {GetSkillLineXPInfo(3,3)}		-- Heavy Armor
		},
		[2] = {	
			[1] = {GetSkillLineXPInfo(2,1)},	-- Two Handed
			[2] = {GetSkillLineXPInfo(2,2)},	-- One Handed & Shield
			[3] = {GetSkillLineXPInfo(2,3)},	-- Dual Wield
			[4] = {GetSkillLineXPInfo(2,4)},	-- Bow
			[5] = {GetSkillLineXPInfo(2,5)},	-- Destruction Staff
			[6] = {GetSkillLineXPInfo(2,6)}		-- Restoration Staff
		},
		[1]	= {	
			[1] = {GetSkillLineXPInfo(1,1)},	-- Class Tree 1
			[2] = {GetSkillLineXPInfo(1,2)},	-- Class Tree 2
			[3] = {GetSkillLineXPInfo(1,3)}		-- Class Tree 3
		}
	}

	-- this is a more i18n way to do it
	CLog.SkillNames = {
		[8] = {	
			[1] = GetSkillLineInfo(8,1) or "Alchemy",
			[2] = GetSkillLineInfo(8,2) or "Blacksmithing",
			[3] = GetSkillLineInfo(8,3) or "Clothing",
			[4] = GetSkillLineInfo(8,4) or "Enchanting",
			[5] = GetSkillLineInfo(8,5) or "Provisioning",
			[6] = GetSkillLineInfo(8,6) or "Woodworking",
		},
		[7] = { [1] = GetSkillLineInfo(7,1)}, -- EXP -- not sure if this is used anymore
		[6] = {
			[1] = GetSkillLineInfo(6,1) or "Assault",
			[2] = GetSkillLineInfo(6,2) or "Support",
			[3] = GetSkillLineInfo(6,3) or "Emperor",
		},
		[5] = {	
			[1] = GetSkillLineInfo(5,1) or "Guild 1",
			[2] = GetSkillLineInfo(5,2) or "Guild 2",
			[3]	= GetSkillLineInfo(5,3) or "Guild 3",
			[4]	= GetSkillLineInfo(5,4) or "Guild 4",
			[5]	= GetSkillLineInfo(5,5) or "Guild 5",
		},
		[4] = {
			[1] = GetSkillLineInfo(4,1) or "Legerdemain",
			[2] = GetSkillLineInfo(4,2) or "World",
			[3] = GetSkillLineInfo(4,3) or "Beast Form",
		},
		[3]	= {	
			[1] = GetSkillLineInfo(3,1) or "Light Armor",
			[2] = GetSkillLineInfo(3,2) or "Medium Armor",
			[3] = GetSkillLineInfo(3,3) or "Heavy Armor",
		},
		[2] = {	
			[1] = GetSkillLineInfo(2,1) or "Two Handed",
			[2] = GetSkillLineInfo(2,2) or "One Handed & Shield",
			[3] = GetSkillLineInfo(2,3) or "Dual Wield",
			[4] = GetSkillLineInfo(2,4) or "Bow",
			[5] = GetSkillLineInfo(2,5) or "Destruction Staff",
			[6] = GetSkillLineInfo(2,6) or "Restoration Staff",
		},
		[1]	= {	
			[1] = GetSkillLineInfo(1,1), -- Class skill line 1
			[2] = GetSkillLineInfo(1,2), -- Class skill line 2
			[3] = GetSkillLineInfo(1,3), -- Class skill line 3
		}
	}
end

--[[----------------------------------------------------------
	Combat Event Handler
 ]]-----------------------------------------------------------

 --[[
  * Handles combat events and adds them to the combat damage table.
  * Runs on the EVENT_COMBAT_EVENT listener.
 ]]--
 function CLog.NewCombatEvent( eventCode , result , isError , abilityName, abilityGraphic, abilityActionSlotType, sourceName, sourceType, targetName, targetType, hitValue, powerType, damageType, log )
	-- Display all combat events for debugging
	--d( "[" ..result .. "]," .. "[" ..abilityName .. "]," .. "[" ..sourceName .. "]," .. "[" ..sourceType .. "]," .. "[" ..targetName .. "]," .. "[" ..targetType .. "]," .. "[" ..hitValue .. "]," .. "[" ..abilityActionSlotType .. "]," .. "[" ..powerType .. "]," .. "[" ..damageType .. "]" )

	if (CLog.State == false) and (CLog.Check == false) then return end

	-- Verify it's a valid result type
	if ( not CLog.FilterSCT( result , abilityName , sourceType , targetName ) ) then return end
	if ( hitValue == 0 ) then return end

	-- Setup a new SCT object
	local newSCT = {
		["source"]		= sourceName,
		["sourceType"]	= sourceType,
		["targetType"]	= targetType,
		["target"]		= targetName,
		["name"]		= abilityName,
		["dam"]			= hitValue,
		["power"]		= powerType,
		["type"]		= damageType,
		["ms"]			= GetGameTimeMilliseconds(),
		["killblow"] 	= ( result == ACTION_RESULT_KILLING_BLOW ),

		["ifheal"] 		= ( result == ACTION_RESULT_HEAL or result == ACTION_RESULT_CRITICAL_HEAL or result == ACTION_RESULT_HOT_TICK_CRITICAL or result == ACTION_RESULT_HOT_TICK ),
		["crit"]		= ( result == ACTION_RESULT_CRITICAL_DAMAGE or result == ACTION_RESULT_CRITICAL_HEAL or result == ACTION_RESULT_HOT_TICK_CRITICAL or result == ACTION_RESULT_DOT_TICK_CRITICAL ),
		["energy"]		= ( result == ACTION_RESULT_POWER_ENERGIZE ),

		["critdam"]		= ( result == ACTION_RESULT_CRITICAL_DAMAGE ),
		["critheal"] 	= ( result == ACTION_RESULT_CRITICAL_HEAL ),
		["critdot"]		= ( result == ACTION_RESULT_DOT_TICK_CRITICAL ),
		["heal"]		= ( result == ACTION_RESULT_HEAL ),
		["hot"]			= ( result == ACTION_RESULT_HOT_TICK ),
		["crithot"]		= ( result == ACTION_RESULT_HOT_TICK_CRITICAL ),
		["dot"]			= ( result == ACTION_RESULT_DOT_TICK ),

		["blockeddam"]	= ( result == ACTION_RESULT_BLOCKED_DAMAGE ),
		["blocked"]		= ( result == ACTION_RESULT_BLOCKED ),
		["immune"]		= ( result == ACTION_RESULT_IMMUNE ),
		["miss"]		= ( result == ACTION_RESULT_MISS ),
		["parried"]		= ( result == ACTION_RESULT_PARRIED or result == ACTION_RESULT_BLADETURN ),
		["resist"]		= ( result == ACTION_RESULT_RESIST ),
		["reflect"]		= ( result == ACTION_RESULT_REFLECTED ),
		["dodge"]		= ( result == ACTION_RESULT_DODGED ),
		["absorb"]		= ( result == ACTION_RESULT_DAMAGE_SHIELDED or result == ACTION_RESULT_ABSORBED ),

		["interrupt"]	= ( result == ACTION_RESULT_INTERRUPT ),
		["disarm"]		= ( result == ACTION_RESULT_DISARMED ),
		["disorient"]	= ( result == ACTION_RESULT_DISORIENTED ),
		["fear"]		= ( result == ACTION_RESULT_FEARED ),
		["knockback"]	= ( result == ACTION_RESULT_KNOCKBACK ),
		["levitate"]	= ( result == ACTION_RESULT_LEVITATED ),
		["offbalance"]	= ( result == ACTION_RESULT_OFFBALANCE ),
		["root"]		= ( result == ACTION_RESULT_ROOTED ),
		["silence"]		= ( result == ACTION_RESULT_SILENCED ),
		["stagger"]		= ( result == ACTION_RESULT_STAGGERED ),
		["stun"]		= ( result == ACTION_RESULT_STUNNED ),
	}
	--d("Before Source=" .. sourceType)
	--d("Before Target=" .. targetType)

	-- Add Damage or Healing to Log
	if (newSCT.energy == false) then
		if (newSCT.ifheal == false) and (CLog.settings.ShowDamage == true) then
			if (sourceType == 1) or (targetType == 1) or (sourceType == 2) or (targetType == 2) then
			--d("Source=" .. sourceType)
			--d("Target=" .. targetType)
			CLog.UpdateCL( newSCT )
			end
		elseif (newSCT.ifheal == true) and (CLog.settings.ShowHeals == true) then
			if (sourceType == 1) or (targetType == 1) or (sourceType == 2) or (targetType == 2) then
			--d("Source=" .. sourceType)
			--d("Target=" .. targetType)
			CLog.UpdateCL( newSCT )
			end
		end
	end
	-- Energy Gains
	if (targetType == 1 and newSCT.energy == true) then
		CLog.UpdateCL( newSCT )
	end

	-- to Damage Meters
	if (newSCT.interrupt ~= true and newSCT.absorb ~= true and newSCT.blocked ~= true and newSCT.resist ~= true and newSCT.dodge ~= true and newSCT.energy ~= true) then
		if (sourceType == 1 and newSCT.ifheal == false) or (sourceType == 2 and newSCT.ifheal == false)then
			CLog.DPS( newSCT.dam , newSCT.name, newSCT.crit, newSCT.type)
		end
		if (sourceType == 1 and targetType == 1) or (sourceType == 1 and newSCT.ifheal == true) or (sourceType == 2 and targetType == 1) or (sourceType == 2 and newSCT.ifheal == true) then
			CLog.HPS( newSCT.dam , newSCT.name, newSCT.crit, newSCT.type )
		end
	end
	if (sourceType == 1 and newSCT.killblow == true) or (sourceType == 2 and newSCT.killblow == true) then
		CLog.KillBlow()
	end
end

--[[-----------------------------------------------
	Buff Event Handler
 ]]------------------------------------------------

--[[
 * Set up buffs which are currently active on the target
 * Runs when the addon is first loaded for the player
 * Runs whenever the reticle target changes
 ]]--
function CLog.GetBuffs( target )
	-- Get the number of buffs currently affecting the target
	local nbuffs = GetNumBuffs( target )

	-- Iterate through them, adding them to the active buffs table
	for i = 1 , nbuffs , 1 do
		local buffName, timeStarted, timeEnding, buffSlot, stackCount, iconFilename, buffType, effectType, abilityType, statusEffectType, abilityId, canClickOff = GetUnitBuffInfo( target , i )
		CLog.UpdateBuff( 1, buffName , buffSlot , timeStarted , timeEnding , stackCount, abilityType, effectType, iconFilename )
	end	
end

--[[
 * This is a wrapper function for processing new active buffs.
 * It runs on the EVENT_EFFECT_CHANGED listener.
 * Changes are passed through the AddBuff function.
 ]]--
function CLog.NewBuff( eventCode, changeType, effectSlot, effectName, unitTag, beginTime, endTime, stackCount, iconName, buffType, effectType, abilityType, statusEffectType )
	if CLog.settings.Buffs == false then return end
	--d( eventCode, changeType, effectSlot, effectName, unitTag, beginTime, endTime, stackCount, iconName, buffType, effectType, abilityType, statusEffectType )

	-- Update the active buffs
	--if (effectName == CLog.PlayerBuffs.[effectName].name) and (CLog.PlayerBuffs.[effectName].start == beginTime) and (CLog.PlayerBuffs.[effectName].start == endTime) then return end
	if (unitTag == "player") and (abilityType ~= ABILITY_TYPE_HEAL) then
		CLog.UpdateBuff( changeType, effectName, effectSlot, beginTime, endTime, stackCount, abilityType, effectType, iconName )
	end
end

--[[
 * This function handles updates to buffs.
 * New buffs, changes to buffs, and expired buffs are passed through.
 ]]--
function CLog.UpdateBuff( changeType, effectName, effectSlot, beginTime, endTime, stackCount, abilityType, effectType, icon )
	if CLog.settings.Buffs == false then return end

	-- The three orb effects are constantly being spammed if you are the wearer, so they require filtering
	if effectName == "Worm's Raiment" or
	   effectName == "Hircine's Veneer" or
	   effectName == "Ebon Armory" then
		return
	end

	-- Debugging all registered abilities
	--d( eventCode .. "/" .. changeType .. "/" .. effectSlot .. "/" .. effectName .. "/" .. unitTag .. "/" .. beginTime .. "/" .. endTime .. "/" .. stackCount .. "/" .. iconName .. "/" .. buffType .. "/" .. effectType .. "/" .. abilityType .. "/" .. statusEffectType )

	-- Keeps track of buffs so that they are not repeated
	local newBuff= {
		["name"] = string.gsub(effectName, "%s+", ""),
		["startT"] = beginTime,
		["endT"] = endTime,
		["icon"] = icon,
	}
	if (CLog.PlayerBuffs[effectName] == nil) then
		-- new buff
		CLog.PlayerBuffs[effectName] = newBuff
	elseif (CLog.PlayerBuffs[effectName].startT ~= beginTime) and (CLog.PlayerBuffs[effectName].endT ~= endTime) then
		-- update for an existing buff
		CLog.PlayerBuffs[effectName] = newBuff
	else return end -- matches info already recorded

	-- Get Output Box
	local display = " "

	-- Run the effect through a filter
	local isValid, effectName, effectDuration , beginTime , endTime = CLog.FilterBuffInfo( effectName , abilityType , beginTime , endTime, effectSlot )
	--d(effectName .. "/ " .. beginTime .. " / " .. endTime .. " / " .. changeType)

	-- Drop ignored effects
	if ( isValid == false ) then return end

	local name = string.gsub(effectName, "%s+", "")

	-- Add new effects or refreshed effects to the active buffs table
	if ( 1 == changeType ) or ( 3 == changeType ) then
		if ( stackCount == nil or 0) then
			display = "You gain " .. effectName
		else
			display = "You gain " .. stackCount .. " of " .. effectName
		end
	-- Remove expired effects from the active buffs table
	elseif ( 2 == changeType ) then
		display = effectName .. " fades from you"
	end

	-- Format and send to Output box
	if CLog.settings.BuffColor == true then
		if (effectType == BUFF_EFFECT_TYPE_NOT_AN_EFFECT) then            -- Food Buff
			display = CLog.settings.FoodBuffColor .. display .. "|r"
		elseif (effectType == BUFF_EFFECT_TYPE_BUFF) then -- Good Buff
			display = CLog.settings.GoodBuffColor .. display .. "|r"
		else                                                   -- Bad Buff
			display = CLog.settings.BadBuffColor .. display .. "|r"
		end
	end

	CLog.AddTextToLog("You", display)
end

--[[-----------------------------------------------
	Check to see if in Combat
 ]]------------------------------------------------
function CLog.InCombat( eventCode , CombatStatus)
	CLog.Check = CombatStatus
	--d(CLog.Check)
	--d(CLog.State)
end

--[[-----------------------------------------------
	Sets Up Text for Combat Log
 ]]------------------------------------------------
function CLog.UpdateCL( CombatData )
	--d(CombatData)

	if (CLog.settings == nil) then
		CLog.settings.You = {"Loaded"}
		CLog.settings.Dam = {"Loaded"}
	end	

--	local CLSV = _G["CombatLog_SliderV"]
--	local CLSH = _G["CombatLog_SliderH"]
	local display = ""
	local damage = CombatData.dam
	local type = CombatData.type
	local source = CombatData.source
	local sType = CombatData.sourceType
	local tType = CombatData.targetType
	local target = CombatData.target
	local time = CombatData.ms
	local ability = CombatData.name
	local DisTable = {}

	-- health
	local percent = math.floor( ( CLog.current / CLog.maximum ) * 100 )

	-- Removes "^" and following characters from targets name
	source = zo_strformat(SI_TOOLTIP_ITEM_NAME, source)

	-- Removes "^" and following characters from targets name
	target = zo_strformat(SI_TOOLTIP_ITEM_NAME, target)

	if (source ~= "") and (ability ~= "") then
		-- here, we're mostly dealing with stuff you do to yourself, stuff you originate, or stuff your pet originates. 
		if CLog.settings.NameColor == true then
			-- Change color of source and target
			-- source
			if (sType == 1) then									-- Your Character
				source = CLog.settings.YouColor .. "You" .. "|r"
			elseif ( sType == 2) then								-- Pet
				source = CLog.settings.PetColor .. source .. "|r"
			elseif ( sType == 3) then								-- Your Group Members
				source = CLog.settings.GroupColor .. source .. "|r"
			elseif ( sType == 4) and ( tType == 1) and (CombatData.heal == false) and (CombatData.hot == false) and (CombatData.crithot == false) then -- Enemy Player
				source = CLog.settings.EnemyColor .. source .. "|r"
			elseif ( sType == 4) then								-- Friendly Player
				source = CLog.settings.AllyColor .. source .. "|r"
			else													-- Enemy NPC
				source = CLog.settings.EnemyColor .. source .. "|r"
			end

			-- target
			if (tType == 1) then									-- Your Character
				target = CLog.settings.YouColor .. "You" .. "|r"
			elseif (tType == 2) then								-- Pet
				sourse = CLog.settings.PetColor .. target .. "|r"
			elseif (tType == 3) then								-- Your Group Members
				target = CLog.settings.GroupColor .. target .. "|r"
			elseif (tType == 4) and (CombatData.heal == false) and (CombatData.hot == false) and (CombatData.crithot == false) then -- Enemy Player
				target = CLog.settings.EnemyColor .. target .. "|r"
			elseif (tType == 4) then								-- Friendly Player
				target = CLog.settings.AllyColor .. target .. "|r"
			else													-- Enemy NPC
				target = CLog.settings.EnemyColor .. target .. "|r"
			end
		else
			-- no pretty colors here
			if (sType == 1) then
				source = "You"
			end
			if (tType == 1) then
				target = "You"
			end
		end

		if CLog.settings.AbilityColor == true then
		-- Change Color of Spell
			if (type == DAMAGE_TYPE_GENERIC) then
				if (CombatData.heal == true or CombatData.critheal == true or CombatData.hot == true or CombatData.crithot == true) then	--Heal
					ability = CLog.settings.HealColor .. ability .. "|r"
				elseif (CombatData.energy ~= true) then
					ability = CLog.settings.GenericColor .. ability .. "|r"
				end
			elseif (type == DAMAGE_TYPE_PHYSICAL) then
				ability = CLog.settings.PhysicalColor .. ability .. "|r"
			elseif (type == DAMAGE_TYPE_FIRE) then
				ability = CLog.settings.FireColor .. ability .. "|r"
			elseif (type == DAMAGE_TYPE_SHOCK) then
				ability = CLog.settings.ShockColor .. ability .. "|r"
			elseif (type == DAMAGE_TYPE_OBLIVION) then
				ability = CLog.settings.OblivionColor .. ability .. "|r"
			elseif (type == DAMAGE_TYPE_COLD) then
				ability = CLog.settings.ColdColor .. ability .. "|r"
			elseif (type == DAMAGE_TYPE_EARTH) then
				ability = CLog.settings.EarthColor .. ability .. "|r"
			elseif (type == DAMAGE_TYPE_MAGIC) then
				ability = CLog.settings.MagicColor .. ability .. "|r"
			elseif (type == DAMAGE_TYPE_DROWN) then
				ability = CLog.settings.DrownColor .. ability .. "|r"
			elseif (type == DAMAGE_TYPE_DISEASE) then
				ability = CLog.settings.DiseaseColor .. ability .. "|r"
			elseif (type == DAMAGE_TYPE_POISON) then
				ability = CLog.settings.PoisonColor .. ability .. "|r"
			else --None
			end
		end

		if (CombatData.heal == true) then
			-- Flag heals
			if string.match( ability , "Potion" ) then
				ability = "Health Potion"
				display = source .. " used " .. ability .. " and Healed " .. damage .. " on " .. target
			else
				if (tType == 0) then
					-- Friendly NPC
					target = CLog.settings.NPCColor .. CombatData.target .. "|r"
				end
				display = source .. " used " .. ability .. " and Healed " .. damage .. " on " .. target
			end
		elseif (CombatData.critheal == true) then
			-- Flag Crit Heal
			if string.match( ability , "Potion" ) then
				ability = "Health Potion"
				display = source .. " used " .. ability .. " and Crit Healed " .. damage .. " on " .. target
			else
				if (tType == 0) then
					-- Friendly NPC
					target = CLog.settings.NPCColor .. CombatData.target .. "|r"
				end
				display = source .. " used " .. ability .. " and " .. CLog.settings.SpecialColor .. "CRIT|r Healed " .. damage .. " on " .. target
			end
		elseif (CombatData.hot == true) then
			-- Flag HOT
			if (tType == 0) then
				-- Friendly NPC
				target = CLog.settings.NPCColor .. CombatData.target .. "|r"
			end
			display = source .. " used " .. ability .. " HoT tick " .. damage .. " on " .. target
		elseif (CombatData.crithot == true) then
			-- Flag Crit HOT
			if (tType == 0) then
				-- Friendly NPC
				target = CLog.settings.NPCColor .. CombatData.target .. "|r"
			end
			display = source .. " used " .. ability .. CLog.settings.SpecialColor .. " CRIT HoT tick|r " .. damage .. " on " .. target
		elseif (CombatData.critdam == true) then
			-- Flag Crit Damage
			display = source .. CLog.settings.SpecialColor .. " CRIT|r " .. target .. " for " .. damage .. " with " .. ability

		elseif (CombatData.blockeddam == true) then
			-- Flag blocked damage
			display = target .. CLog.settings.SpecialColor .. " BLOCKED|r " .. source .. " and took " .. damage .. " from " .. ability
		elseif (CombatData.immune == true) then
			-- Flag damage immunity
			display = source .. " is immune from " .. ability
		elseif (CombatData.parried == true) then
			-- Flag damage parried
			display = source .. CLog.settings.SpecialColor .. " PARRIED|r " .. target .. "'s " .. ability
		elseif (CombatData.miss == true) then
			-- Flag damage missed
			display = source .. CLog.settings.SpecialColor .. " MISSED|r " .. target .. "'s " .. ability
		elseif (CombatData.reflect == true) then
			-- Flag damage reflect
			display = source .. CLog.settings.SpecialColor .. " REFLECTED|r " .. target .. "'s " .. ability
		elseif (CombatData.resist == true) then
			-- Flag damage resist
			display = source .. CLog.settings.SpecialColor .. " RESISTED|r " .. target .. "'s " .. ability
		elseif (CombatData.dodge == true) then
			-- Flag damage dodged
			display = source .. CLog.settings.SpecialColor .. " DODGED|r " .. target .. "'s " .. ability
		elseif (CombatData.absorb == true) then
			-- Flag damage absorbed
			display = target .. CLog.settings.SpecialColor .. " ABSORBED|r " .. damage .. " with " .. ability

		elseif (CombatData.interrupt == true) then
			-- Flag inturrupted
			display = source .. CLog.settings.SpecialColor .. " INTERRUPTED|r " .. target .. " with " .. ability
		elseif (CombatData.disorient == true) then
			-- Flag disorientations
			display = source .. CLog.settings.SpecialColor .. " DISORIENTED|r " .. target .. " with " .. ability
		elseif (CombatData.fear == true) then
			-- Flag fears
			display = source .. CLog.settings.SpecialColor .. " FEARED|r " .. target .. " with " .. ability
		elseif (CombatData.knockback == true) then
			-- Flag knockbacks
			display = source .. CLog.settings.SpecialColor .. " KNOCKED BACK|r " .. target .. " with " .. ability
		elseif (CombatData.levitate == true) then
			-- Flag levitates
			display = source .. CLog.settings.SpecialColor .. " LEVITATED|r " .. target .. " with " .. ability
		elseif (CombatData.offbalance == true) then
			-- Flag off-balance
			display = source .. " threw " .. target .. CLog.settings.SpecialColor .. " OFF BALANCE|r with " .. ability
		elseif (CombatData.root == true) then
			-- Flag rooted
			display = source .. CLog.settings.SpecialColor .. " ROOTED|r " .. target .. " with " .. ability
		elseif (CombatData.silence == true) then
			-- flag silenced
			display = source .. CLog.settings.SpecialColor .. " SILENCED|r " .. target .. " with " .. ability
		elseif (CombatData.stagger == true) then
			-- Flag staggered
			display = source .. CLog.settings.SpecialColor .. " STAGGERED|r " .. target .. " with " .. ability
		elseif (CombatData.stun == true) then
			-- Flag stunned
			display = source .. CLog.settings.SpecialColor .. " STUNNED|r " .. target .. " with " .. ability

		elseif (CombatData.energy == true) then
			-- Flag STAM/MAGIC/ULTIMAT Gains
			if (CombatData.power == POWERTYPE_MAGICKA) then
				-- Magicka
				display = "|c4542ff+" .. damage .. " Magicka with " .. ability .. "|r"
			elseif (CombatData.power == POWERTYPE_STAMINA) then
				-- Stamina
				display = "|c08db19+" .. damage .. " Stamina with " .. ability .. "|r"
			elseif (CombatData.power == POWERTYPE_ULTIMATE) then
				-- Ultimate
				display = "|cffe862+" .. damage .. " Ultimate with " .. ability .. "|r"
			end

		elseif (CombatData.dot == true) then
			-- Flag damage DoT
			display = source .. " DoT tick " .. target .. " for " .. damage .. " with " .. ability
		elseif (CombatData.critdot == true) then
			-- Flag damage DoT CRIT
			display = source .. CLog.settings.SpecialColor .. " CRIT DoT tick|r " .. target .. " for " .. damage .. " with " .. ability
		else
			-- Standard hits
			display = source .. " hit " .. target .. " for " .. damage .. " with " .. ability
		end
	else
		-- if we don't know the source or the ability name, then it's incoming
		if (CombatData.ifheal == false) then
			CLog.settings.Stats.IncDam = CLog.settings.Stats.IncDam + damage
			if damage > CLog.settings.Stats.MaxIncDam then
				CLog.settings.Stats.MaxIncDam = damage
			end

			if (CombatData.blockeddam == true or CombatData.blocked == true) then
				display = CLog.settings.YouColor .. "You|r" .. CLog.settings.SpecialColor .. " BLOCKED|r " .. damage
			elseif (CombatData.interrupt == true) then
				display = CLog.settings.YouColor .. "You|r were " .. CLog.settings.SpecialColor .. "INTERRUPTED|r"
			elseif (CombatData.disorient == true) then
				display = CLog.settings.YouColor .. "You|r became " .. CLog.settings.SpecialColor .. "DISORIENTED|r"
			elseif (CombatData.fear == true) then
				display = CLog.settings.YouColor .. "You|r were " .. CLog.settings.SpecialColor .. "FEARED|r"
			elseif (CombatData.knockback == true) then
				display = CLog.settings.YouColor .. "You|r were " .. CLog.settings.SpecialColor .. "KNOCKED BACK|r"
			elseif (CombatData.levitate == true) then
				display = CLog.settings.YouColor .. "You|r were " .. CLog.settings.SpecialColor .. "LEVITATED|r"
			elseif (CombatData.offbalance == true) then
				display = CLog.settings.YouColor .. "You|r were thrown " .. CLog.settings.SpecialColor .. "OFF BALANCE|r"
			elseif (CombatData.root == true) then
				display = CLog.settings.YouColor .. "You|r were " .. CLog.settings.SpecialColor .. "ROOTED|r"
			elseif (CombatData.silence == true) then
				display = CLog.settings.YouColor .. "You|r were " .. CLog.settings.SpecialColor .. "SILENCED|r"
			elseif (CombatData.stagger == true) then
				display = CLog.settings.YouColor .. "You|r were " .. CLog.settings.SpecialColor .. "STAGGERED|r"
			elseif (CombatData.stun == true) then
				display = CLog.settings.YouColor .. "You|r were " .. CLog.settings.SpecialColor .. "STUNNED|r"
			else
				display = CLog.settings.YouColor .. "You|r were hit for " .. damage
			end
		else
			CLog.settings.Stats.IncHeal = CLog.settings.Stats.IncHeal + damage
			if damage > CLog.settings.Stats.MaxIncHeal then
				CLog.settings.Stats.MaxIncHeal = damage
			end
			display = CLog.settings.YouColor .. "You|r were healed for " .. damage
		end
	end

	-- Save Data into Table	
	if (tType == 1) and (CombatData.energy == false) then
		if (percent == 0) then
			CLog.AddTextToLog("Dam", "Dead")
		else
			CLog.AddTextToLog("Dam", percent .. "% ( " .. CLog.current .. " ) - " .. display)
		end
	end

	CLog.AddTextToLog("You", display)
end

--[[-----------------------------------------------
	Finesse Rank Event Handler
 ]]------------------------------------------------
--function CLog.Finesse(eventcode, target, finesseType, text, source, log)
	--d(eventcode .. "/" .. target .. "/" .. targetType .. "/" .. text .. "/" .. source)
	--local finesse = _G["CombatLog_Finesse"]
	--if (target == "player") then
		--finesse:SetText("|cead585" .. text .. "|r")
	--end
--end

--[[----------------------------------------------------------
	Combat FILTER FUNCTIONS
 ]]-----------------------------------------------------------

--[[
 * Filter combat events to validate including them in SCT
 ]]--
function CLog.FilterSCT( result , abilityName , sourceType , targetName )
	targetName = string.gsub(targetName, "%^.*", "")
	if (result == ACTION_RESULT_DAMAGE or result == ACTION_RESULT_CRITICAL_DAMAGE or result == ACTION_RESULT_BLOCKED_DAMAGE or result == ACTION_RESULT_FALL_DAMAGE) then
		-- Direct Damage
		return true
	elseif (result == ACTION_RESULT_DOT_TICK or result == ACTION_RESULT_DOT_TICK_CRITICAL) then
		-- Damage Over Time
		return true
	elseif (result == ACTION_RESULT_HEAL or result == ACTION_RESULT_CRITICAL_HEAL or result == ACTION_RESULT_HOT_TICK or result == ACTION_RESULT_HOT_TICK_CRITICAL) then
		-- Heals
		return true
	elseif (result == ACTION_RESULT_IMMUNE
		or result == ACTION_RESULT_BLOCKED
		or result == ACTION_RESULT_DODGED
		or result == ACTION_RESULT_REFLECTED
		or result == ACTION_RESULT_RESIST
		or result == ACTION_RESULT_PARRIED
		or result == ACTION_RESULT_MISS
		or result == ACTION_RESULT_DAMAGE_SHIELDED
		or result == ACTION_RESULT_ABSORBED
		or result == ACTION_RESULT_BLADETURN) then
		-- Damage Immunity
		return true
	elseif (result == ACTION_RESULT_DISARMED
		or result == ACTION_RESULT_INTERRUPT
		or result == ACTION_RESULT_DISORIENTED
		or result == ACTION_RESULT_FEARED
		or result == ACTION_RESULT_KNOCKBACK
		or result == ACTION_RESULT_LEVITATED
		or result == ACTION_RESULT_OFFBALANCE
		or result == ACTION_RESULT_ROOTED
		or result == ACTION_RESULT_SILENCED
		or result == ACTION_RESULT_STAGGERED
		or result == ACTION_RESULT_STUNNED) then
		-- non-damage combat actions
		return true
	elseif (CLog.settings.ShowPower == true) and (result == ACTION_RESULT_POWER_ENERGIZE) then
		-- Power attack charge
		return true
	else
		-- Otherwise, ignore it
		return false
	end	
end

--[[----------------------------------------------------------
	Buff Filter Function
 ]]-----------------------------------------------------------	
--[[
 * Filter abilities to override their displayed names or durations as necessary
 ]]--
function CLog.FilterBuffInfo( name , buffType , beginTime , endTime, effectSlot )
	-- Default to no duration
	local duration 	= nil
	local isValid	= true

	if ( string.match( name , 'TargetingHit' ) or string.match( name , 'Hack' ) or string.match( name , 'dummy' ) or string.match( name , "Mount Up" ) or string.match( name , "Boon: " ) or string.match( name , "Force Siphon" ) or string.match( name , "Regeneration" )) then
		-- Effects to ignore
		isValid		= false
	elseif ( name == "Unstable Clannfear" or name == "Unstable Familiar" or name == "Volatile Familiar" ) then
		-- Summons
		duration 	= "Summon"
	elseif ( name == "Siphoning Strikes" ) or ( name == "Magelight" ) then
		-- Toggles
		duration 	= "Toggle"	
	elseif ( buffType == ABILITY_TYPE_BONUS ) then
		-- "Bonus" Abilities
		-- Ignore Cyrodiil Keep Bonuses
		if ( string.match( name , "Keep Bonus" ) ) then
			isValid		= false
		end
	elseif ( buffType == ABILITY_TYPE_STUN ) then
		-- Stunned (9)
		name		= "Stunned"
	elseif ( buffType == ABILITY_TYPE_STAGGER ) then
		-- Power Attack (33)
		if ( beginTime == 0 ) then
			isValid		= false
		else
			name		= "Staggered"
			endTime		= beginTime + 3
		end
	elseif ( ( buffType == ABILITY_TYPE_BLOCK ) or ( name == "Brace (Generic)" ) ) then
		-- Blocking (52)
		isValid = false
	elseif ( buffType == ABILITY_TYPE_OFFBALANCE ) then
		-- Off-Balance (53)
		endTime		= beginTime + 3
	elseif ( string.match( name , "^.-Potion" ) ) then
		-- Potions
		name		= string.match( name , '^.-Potion' )
	end

	return isValid, name, duration , beginTime , endTime
end