local BufferTable = {}
local function BufferReached(key, buffer)
	if key == nil then return end
	if BufferTable[key] == nil then BufferTable[key] = {} end
	BufferTable[key].buffer = buffer or 3
	BufferTable[key].now = GetFrameTimeSeconds()
	if BufferTable[key].last == nil then BufferTable[key].last = BufferTable[key].now end
	BufferTable[key].diff = BufferTable[key].now - BufferTable[key].last
	BufferTable[key].eval = BufferTable[key].diff >= BufferTable[key].buffer
	if BufferTable[key].eval then BufferTable[key].last = BufferTable[key].now end
	return BufferTable[key].eval
end

function CLog.LTCreate()
	-- Life Time Stats
		local display = {}
		local dps = CLog.settings.Stats["TDam"]/CLog.settings.Stats["FTime"]
		table.insert(display, "|cff361eDPS:|r " .. string.format( "%.2f" , dps ))
		table.insert(display, "|cff361eMax Damage:|r " .. CLog.settings.Stats["MaxDam"])
		table.insert(display, "|cff361eDamage Done:|r " .. CLog.settings.Stats["TDam"])
		table.insert(display, "|cff361eMax Crit Damage:|r " .. CLog.settings.Stats["MaxCritDam"])
		local hps = CLog.settings.Stats["THeal"]/CLog.settings.Stats["FTime"]
		table.insert(display, "|c00ffffHPS:|r " .. string.format( "%.2f" , hps ))
		table.insert(display, "|c00ffffMax Heal:|r " .. CLog.settings.Stats["MaxHeal"])
		table.insert(display, "|c00ffffHealing Done:|r " .. CLog.settings.Stats["THeal"])
		table.insert(display, "|c00ffffMax Crit Heal:|r " .. CLog.settings.Stats["MaxCritHeal"])
		table.insert(display, "|cff361eMax Dam Taken:|r " .. CLog.settings.Stats["MaxIncDam"])
		table.insert(display, "|cff361eDamage Taken:|r " .. CLog.settings.Stats["IncDam"])
		table.insert(display, "|c00ffffMax Heal Taken:|r " .. CLog.settings.Stats["MaxIncHeal"])
		table.insert(display, "|c00ffffHealing Taken:|r " .. CLog.settings.Stats["IncHeal"])
		table.insert(display, "|cff361eKilling Blows:|r " .. CLog.settings.Stats["Kills"])
		table.insert(display, "|cff361eDeaths:|r " .. CLog.settings.Stats["Death"])
		return display
end

--[[----------------------------------------------------------
	Sorts Table
 ]]-----------------------------------------------------------
function CLog.SortTable(Itable)
	local HTable = {}
	local DTable = {}
	local STable = {}
	local MTable = {}
	local MDTable = {}
	local Tdam = 1
	local Scolor = {}

	if Itable ~= nil then
		for k, v in pairs(Itable) do
			HTable[k] = Itable[k]["Total Damage"]	-- HTable["Ability Name"] = integer damage
		end
		for k , v in pairs(HTable) do
			table.insert(STable, k)					-- STable[index] = "Ability Name"
		end
		local function sortMyTable(a, b)
			return HTable[a] > HTable[b]
		end

		table.sort(STable, sortMyTable)

		for i, ability in ipairs(STable) do
			local color = Itable[ability]["Color"]	-- Color is actually damage type number
			local ctype = ""						-- ctype is the color string
			local dtype = ""						-- dtype is a text description of the damage type

			-- Change Color of ability according to damage type
				if (color == 1) then						--Generic	
					if CLog.currentTab == "Heal" then		--Heal
						ctype = CLog.settings.HealColor
						dtype = "Healing"
					else
						ctype = CLog.settings.GenericColor
						dtype = "Generic"
					end
				elseif (color == 2) then					--Physical
					ctype = CLog.settings.PhysicalColor
					dtype = "Physical"
				elseif (color == 3) then					--Fire
					ctype = CLog.settings.FireColor
					dtype = "Fire"
				elseif (color == 4) then					--Shock
					ctype = CLog.settings.ShockColor
					dtype = "Shock"
				elseif (color == 5) then					--Oblivion
					ctype = CLog.settings.OblivionColor
					dtype = "Oblivion"
				elseif (color == 6) then					--Cold
					ctype = CLog.settings.ColdColor
					dtype = "Frost"
				elseif (color == 7) then					--Earth
					ctype = CLog.settings.EarthColor
					dtype = "Earth"
				elseif (color == 8) then					--Magic
					ctype = CLog.settings.MagicColor
					dtype = "Magic"
				elseif (color == 9) then					--Drown
					ctype = CLog.settings.DrownColor
					dtype = "Drowning"
				elseif (color == 10) then					--Disease
					ctype = CLog.settings.DiseaseColor
					dtype = "Disease"
				elseif (color == 11) then					--Poison
					ctype = CLog.settings.PoisonColor
					dtype = "Poison"
				else										--None
					dtype = "None"
				end
			if CLog.settings.AbilityColor == false then
				ctype = "|cffffff"
			end

			Scolor[ability] = ctype					-- Scolor["Ability Name"] = "|cffffff"

			table.insert(MTable, Itable[ability])	-- MTable[index] = {"Times Used" = 5, "Total Damage" = 42, etc.}
			MTable[#MTable]["Damage Type"] = dtype
			table.insert(DTable, ctype .. ability .. ":|r " .. HTable[ability])	-- DTable[index] = "|cffffffAbility Name:|r ".. total damage
			Tdam = Tdam + HTable[ability]
		end
	end
	table.insert(DTable, "Total: ".. Tdam)

	if CLog.currentTab == "Attack" then
		for i, v in pairs(MTable) do
			local keytable = {}
			table.insert(keytable, Scolor[STable[i]] .. STable[i] .. " Stats -|r")
			table.insert(keytable, "Type of Damage: " .. Scolor[STable[i]] .. MTable[i]["Damage Type"] .. "|r")
			table.insert(keytable, "Total Damage: " .. MTable[i]["Total Damage"])
			table.insert(keytable, "Times Used: " .. MTable[i]["Times Used"])
			table.insert(keytable, "Number of Crits: " .. MTable[i]["Number Crits"])
			table.insert(keytable, "Max Damage: " .. MTable[i]["Max Damage"])
			table.insert(keytable, "Min Damage: " .. MTable[i]["Min Damage"])
			local per = MTable[i]["Total Damage"] / Tdam * 100
			table.insert(keytable, "Percent of Total Damage: " .. string.format( "%.2f" , per) .. "%")
			local crit = MTable[i]["Number Crits"] / MTable[i]["Times Used"] * 100
			table.insert(keytable, "Percent Crit: " .. string.format( "%.2f" , crit) .. "%")
			table.insert(MDTable, table.concat(keytable, "\n"))		-- MDTable[index] = "Long description of stats for ability"
		end
	elseif CLog.currentTab == "Heal" then
		for i, v in pairs(MTable) do
			local keytable = {}
			table.insert(keytable, Scolor[STable[i]] .. STable[i] .. " Stats -|r")
			table.insert(keytable, "Total Heal: " .. MTable[i]["Total Damage"])
			table.insert(keytable, "Times Used: " .. MTable[i]["Times Used"])
			table.insert(keytable, "Number of Crits: " .. MTable[i]["Number Crits"])
			table.insert(keytable, "Max Heal: " .. MTable[i]["Max Damage"])
			table.insert(keytable, "Min Heal: " .. MTable[i]["Min Damage"])
			local per = MTable[i]["Total Damage"] / Tdam * 100
			table.insert(keytable, "Percent of Total Heal: " .. string.format( "%.2f" , per) .. "%")
			local crit = MTable[i]["Number Crits"] / MTable[i]["Times Used"] * 100
			table.insert(keytable, "Percent Crit: " .. string.format( "%.2f" , crit) .. "%")
			table.insert(MDTable, table.concat(keytable, "\n"))
		end
	end
	if (CLog.settings.Direction == "DOWN") then
		return DTable, MDTable
	else
		local RevDTable = {}
		local RevMDTable = {}
		RevDTable = CLog.TableReverse(DTable)
		RevMDTable = CLog.TableReverse(MDTable)
		return RevDTable, RevMDTable
	end
end

--[[----------------------------------------------------------
	Reverse Table
 ]]-----------------------------------------------------------
function CLog.TableReverse ( tab )
    local size = #tab+1
    local newTable = {}
    for i,v in ipairs ( tab ) do
        newTable[size-i] = v
    end
    return newTable
end

--[[----------------------------------------------------------
	ADD up Damage for DPS Function
 ]]-----------------------------------------------------------
function CLog.DPS( damage , ability, crit, color )
	-- Record Total Damage
	CLog.settings.Stats.TDam = CLog.settings.Stats.TDam + damage
	-- Record Max Damage
	if crit then
		if damage > CLog.settings.Stats.MaxCritDam then
			CLog.settings.Stats.MaxCritDam = damage
		end
	else
		if damage > CLog.settings.Stats.MaxDam then
			CLog.settings.Stats.MaxDam = damage
		end
	end

	local dps = CLog.DPSMeter
	CLog.Damage = CLog.settings.Attack

	-- Add up Damage
	dps = dps + damage

	-- Send data back to the FTC object
	CLog.DPSMeter = dps
	CLog.TotalDamage = CLog.TotalDamage + damage

	-- Adds ability to table
	if CLog.Damage == nil then
		CLog.Damage = {}
	end

	if CLog.Damage[ability] == nil then
		CLog.Damage[ability] = {
			["Total Damage"] = 0,
			["Times Used"] = 0,
			["Number Crits"] = 0,
			["Max Damage"] = 0,
			["Min Damage"] = 1000000000,
			["Color"] = 0
		}
	end
	CLog.Damage[ability]["Color"] = color
	CLog.Damage[ability]["Total Damage"] = CLog.Damage[ability]["Total Damage"] + damage
	CLog.Damage[ability]["Times Used"] = CLog.Damage[ability]["Times Used"] + 1
	if damage > CLog.Damage[ability]["Max Damage"] then
		CLog.Damage[ability]["Max Damage"] = damage
	end
	if damage < CLog.Damage[ability]["Min Damage"] then
		CLog.Damage[ability]["Min Damage"] = damage
	end
	if crit == true then
		CLog.Damage[ability]["Number Crits"] = CLog.Damage[ability]["Number Crits"] + 1
	end	
	-- Save New Data
	CLog.settings.Attack = CLog.Damage	
	-- Display Results
	CLog.ButtonToggle()
end

--[[----------------------------------------------------------
	ADD up Heal for HPS Function
 ]]-----------------------------------------------------------
function CLog.HPS( damage , ability, crit, color )
	-- Record Total Healing
	CLog.settings.Stats.THeal = CLog.settings.Stats.THeal + damage
	-- Record Max Heal
	if crit then
		if damage > CLog.settings.Stats.MaxCritHeal then
			CLog.settings.Stats.MaxCritHeal = damage
		end
	else
		if damage > CLog.settings.Stats.MaxHeal then
			CLog.settings.Stats.MaxHeal = damage
		end
	end

	local hps = CLog.HPSMeter
	-- Add up Damage
	hps = hps + damage		

	-- Send data back to the FTC object
	CLog.HPSMeter = hps
	CLog.TotalHealing = CLog.TotalHealing + damage

	-- Adds ability to table
	if CLog.Heal == nil then
		CLog.Heal = {}
	end

	if CLog.Heal[ability] == nil then
		CLog.Heal[ability] = {
			["Total Damage"] = 0,
			["Times Used"] = 0,
			["Number Crits"] = 0,
			["Max Damage"] = 0,
			["Min Damage"] = 1000000000,
			["Color"] = 0
		}
	end
	CLog.Heal[ability]["Color"] = color
	CLog.Heal[ability]["Total Damage"] = CLog.Heal[ability]["Total Damage"] + damage
	CLog.Heal[ability]["Times Used"] = CLog.Heal[ability]["Times Used"] + 1
	if damage > CLog.Heal[ability]["Max Damage"] then
		CLog.Heal[ability]["Max Damage"] = damage
	end
	if damage < CLog.Heal[ability]["Min Damage"] then
		CLog.Heal[ability]["Min Damage"] = damage
	end
	if crit == true then
		CLog.Heal[ability]["Number Crits"] = CLog.Heal[ability]["Number Crits"] + 1
	end
	-- Save New Data
	CLog.settings.Heal = CLog.Heal
	-- Display Results
	CLog.ButtonToggle()
end

--[[----------------------------------------------------------
	Killing Blow (probably only PvP)
 ]]-----------------------------------------------------------
 function CLog.KillBlow()
	-- Record Killing Blows
	CLog.settings.Stats.Kills = CLog.settings.Stats.Kills + 1
	-- Output to You Window
	if CLog.settings.NameColor == true then
		local display = CLog.settings.YouColor .. "You|r dealt KILLING BLOW (" .. CLog.settings.Kills .. ")"
	else
		local display = "You dealt KILLING BLOW (" .. CLog.settings.Kills .. ")"
	end
	CLog.AddTextToLog("You", display)
 end

--[[----------------------------------------------------------
	Death Happens
 ]]-----------------------------------------------------------
function CLog.Dead()
	-- Count and Record Deaths
	CLog.settings.Stats.Death = CLog.settings.Stats.Death + 1
	if (CLog.Target.lvl == 1) and (CLog.Target.dif == 0) and (CLog.Target.react > 1) then
		local display = "|cFF0000Targeting a Critter When Killed: SERIOUSLY?!?!?! Was that REALLY what killed you????!!!!!|r"
		CLog.AddTextToLog("You", display)
	else
		CLog.AddTextToLog("You", "|cFF0000DEAD|r")
	end
end

--[[----------------------------------------------------------
	DPS Update with time (OnUpdate)
 ]]-----------------------------------------------------------
function CLS_DPSUpdate( self , time )
	--[[-------------------------------------
		Display
	--]]-------------------------------------
	-- Delay Start
	if CLog.Starter == false then return end

	if CLog.settings.Buffer == true then
		--Wykkyd's Buffer
		if not BufferReached("CLSBuffer", CLog.settings.BufferSpeed) then return end
	end

	-- fade the main window if "fade after combat" is set
	if (CLog.settings.HiddenM == false) then
		if ZO_CompassFrame:IsHidden() == true then
			-- never show when the compass is hidden
			CLog.MainWindow:SetHidden(true)
			CLog.MainText:SetHidden(true)
		elseif (CLog.settings.FadesWithChat == true) then
			if IsGameCameraUIModeActive() == true then
				-- always show in UI mode
				CLog.MainWindow:SetHidden(false)
				CLog.MainText:SetHidden(false)
				CLog.FadeTime = time * 1000
			elseif CLog.State == true then
				-- always show when we're in combat
				CLog.MainWindow:SetHidden(false)
				CLog.MainText:SetHidden(false)
			else
				if time < CLog.FadeTime/1000 + CLog.settings.FadeTimer then
					-- wake up if there's been a new entry recently enough
					CLog.MainWindow:SetHidden(false)
					if CLog.settings.FadeTextToo == true then CLog.MainText:SetHidden(false) end
				else
					-- fade if it's been long enough since last change to the log
					CLog.MainWindow:SetHidden(true)
					if CLog.settings.FadeTextToo == true then CLog.MainText:SetHidden(true) end
				end
			end
		else
			CLog.MainWindow:SetHidden(false)
			CLog.MainText:SetHidden(false)
		end
	else
		CLog.MainWindow:SetHidden(true)
		CLog.MainText:SetHidden(true)
	end

	if (CLog.settings.HiddenD == false) then
		if ZO_CompassFrame:IsHidden() == true then
			-- never show when the compass is hidden
			CLog.DPSWindow:SetHidden(true)
		elseif (CLog.settings.FadeDPSWithChat == true) then
			if IsGameCameraUIModeActive() == true then
				-- always show in UI mode
				CLog.DPSWindow:SetHidden(false)
				CLog.FadeTime = time * 1000
			elseif CLog.State == true then
				-- always show when in combat
				CLog.DPSWindow:SetHidden(false)
			else
				-- here we check the timer
				if time < CLog.FadeTime/1000 + CLog.settings.FadeTimer then
					-- show if it hasn't been long enough
					CLog.DPSWindow:SetHidden(false)
				else
					-- hide if it has been long enough
					CLog.DPSWindow:SetHidden(true)
				end
			end
		else
			-- here, the window is set to always on (don't fade)
			CLog.DPSWindow:SetHidden(false)
		end
	else
		-- here, the window is set to off
		CLog.DPSWindow:SetHidden(true)
	end
	
	--[[----------------------------------
		Numbers for Meter
	--]]----------------------------------
		-- Get Player Stats
	CLog.current, CLog.maximum = GetUnitPower( "player" , POWERTYPE_HEALTH )
	local percent = math.floor( ( CLog.current / CLog.maximum ) * 100 )

	-- Remake LT Display if it's shown
	if CLog.currentTab == "LT" then
		CLog.ScrollWheel(CLog.LTCreate())
	end

	if (CLog.State == true) then
		CLog.FadeTime = time * 1000
		-- For Buttons
		if (CLog.DPSMeter == nil) then CLog.DPSMeter = 0 end

		if (CLog.HPSMeter == nil) then CLog.HPSMeter = 0 end

		if (self.StartTime == 0) then self.StartTime = time end

		local startTime = self.StartTime

		-- Calculate fight time
		local fight_time =  math.max(( time - startTime ) , 1 )
		CLog.FTime = fight_time

		-- Calculate DPS
		local dps = CLog.DPSMeter / fight_time

		-- Calculate HPS
		local hps = CLog.HPSMeter / fight_time

		-- Generate Output to Label
		local displayval = string.format( "%.2f" , dps ) .. "\n" .. CLog.DPSMeter .. "\n" .. string.format( "%.2f" , hps ) .. "\n" .. CLog.HPSMeter .. "\n" .. string.format( "%.2f" , fight_time )

		-- Display Results
		CLog.DPSWindow.LabelVal:SetText(displayval)
		CLog.DPSWindow:SetDimensions(DPS_LabelVal:GetWidth()+DPS_LabelTex:GetWidth()+25,DPS_LabelVal:GetHeight() + DPS_Close:GetHeight() + DPS_SendToChat:GetHeight() + 5)

		-- Wykkyd Told me to do it
		CLog.Target = {
			["lvl"] = GetUnitLevel("reticleover"),
			["dif"] = GetUnitDifficulty("reticleover"),
			["react"] = GetUnitReaction("reticleover")
		}
	elseif (CLog.State == false) and (CLog.Check == false) then
		CLog.settings.Stats.FTime = CLog.settings.Stats.FTime + CLog.FTime
		CLog.FTime = 0
		CLog.DPSMeter = 0
		CLog.HPSMeter = 0
		self.StartTime = 0
	end

	CLog.State = IsUnitInCombat('player')
end